import { resolveElements } from '../render/dom/utils/resolve-element.mjs';
import { visualElementStore } from '../render/store.mjs';
import { invariant } from '../utils/errors.mjs';
import { GroupPlaybackControls } from './GroupPlaybackControls.mjs';
import { isDOMKeyframes } from './utils/is-dom-keyframes.mjs';
import { animateTarget } from './interfaces/visual-element-target.mjs';
import { createVisualElement } from './utils/create-visual-element.mjs';
import { animateSingleValue } from './interfaces/single-value.mjs';

function animateElements(elementOrSelector, keyframes, options) {
    const elements = resolveElements(elementOrSelector);
    const numElements = elements.length;
    invariant(Boolean(numElements), "No valid element provided.");
    const animations = [];
    for (let i = 0; i < numElements; i++) {
        const element = elements[i];
        /**
         * Check each element for an associated VisualElement. If none exists,
         * we need to create one.
         */
        if (!visualElementStore.has(element)) {
            /**
             * TODO: We only need render-specific parts of the VisualElement.
             * With some additional work the size of the animate() function
             * could be reduced significantly.
             */
            createVisualElement(element);
        }
        const visualElement = visualElementStore.get(element);
        animations.push(...animateTarget(visualElement, { ...keyframes, transition: options }, {}));
    }
    return new GroupPlaybackControls(animations);
}
function animate(valueOrElement, keyframes, options = {}) {
    if (isDOMKeyframes(keyframes)) {
        return animateElements(valueOrElement, keyframes, options);
    }
    else {
        return animateSingleValue(valueOrElement, keyframes, options);
    }
}

export { animate };
